@@ -67,7 +67,9 @@ extern int errno;
 #ifdef NETATALK
 #include <netatalk/at.h>
 #endif
+#include <arpa/inet.h>
 #include <netinet/in.h>
+#include <ctype.h>
 #include <dlfcn.h>
 #include <err.h>
 #include <grp.h>
@@ -98,6 +100,7 @@ void ktrsockaddr(struct sockaddr *);
 void ktrstat(struct stat *);
 void ktrstruct(char *, size_t);
 void usage(void);
+void sockfamilyname(int);
 const char *ioctlname(u_long);
 
 int timestamp, decimal, fancy = 1, suppressdata, tail, threads, maxdata,
@@ -217,6 +220,7 @@ main(int argc, char *argv[])
 			ktrsysret((struct ktr_sysret *)m);
 			break;
 		case KTR_NAMEI:
+		case KTR_SYSCTL:
 			ktrnamei(m, ktrlen);
 			break;
 		case KTR_GENIO:
@@ -288,6 +292,9 @@ dumpheader(struct ktr_header *kth)
 	case KTR_STRUCT:
 		type = "STRU";
 		break;
+	case KTR_SYSCTL:
+		type = "SCTL";
+		break;
 	default:
 		(void)sprintf(unknown, "UNKNOWN(%d)", kth->ktr_type);
 		type = unknown;
@@ -771,12 +778,6 @@ ktrsyscall(struct ktr_syscall *ktr)
 				mountflagsname ((int)*ip);
 				ip++;
 				narg--;
-			} else if (ktr->ktr_code == SYS_kse_thr_interrupt) {
-				print_number(ip,narg,c);
-				(void)putchar(',');
-				ksethrcmdname ((int)*ip);
-				ip++;
-				narg--;
 			} else if (ktr->ktr_code == SYS_thr_create) {
 				print_number(ip,narg,c);
 				print_number(ip,narg,c);
@@ -1365,6 +1366,6 @@ void
 usage(void)
 {
 	fprintf(stderr, "usage: kdump [-dEnlHRrsT] [-f trfile] "
-	    "[-m maxdata] [-p pid] [-t [cnistuw]]\n");
+	    "[-m maxdata] [-p pid] [-t trstr]\n");
 	exit(1);
 }
