@@ -142,6 +142,8 @@ sbtoxsockbuf(struct sockbuf *sb, struct xsockbuf *xsb)
 	xsb->sb_cc = sb->sb_cc;
 	xsb->sb_hiwat = sb->sb_hiwat;
 	xsb->sb_mbcnt = sb->sb_mbcnt;
+	xsb->sb_mcnt = sb->sb_mcnt;
+	xsb->sb_ccnt = sb->sb_ccnt;
 	xsb->sb_mbmax = sb->sb_mbmax;
 	xsb->sb_lowat = sb->sb_lowat;
 	xsb->sb_flags = sb->sb_flags;
@@ -405,13 +407,22 @@ protopr(u_long off, const char *name, int af1, int proto)
 			if (Lflag)
 				printf("%-5.5s %-14.14s %-22.22s\n",
 				    "Proto", "Listen", "Local Address");
-			else
-				printf((Aflag && !Wflag) ?
-		"%-5.5s %-6.6s %-6.6s  %-18.18s %-18.18s %s\n" :
-		"%-5.5s %-6.6s %-6.6s  %-22.22s %-22.22s %s\n",
-				    "Proto", "Recv-Q", "Send-Q",
-				    "Local Address", "Foreign Address",
-				    "(state)");
+			else {
+				printf((Aflag && !Wflag) ? 
+				       "%-5.5s %-6.6s %-6.6s  %-18.18s %-18.18s" :
+				       "%-5.5s %-6.6s %-6.6s  %-22.22s %-22.22s",
+				       "Proto", "Recv-Q", "Send-Q",
+				       "Local Address", "Foreign Address");
+				if (xflag)
+					printf("%-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %-6.6s %s\n",
+					       	"R-MBUF", "S-MBUF", "R-CLUS", 
+						"S-CLUS", "R-HIWA", "S-HIWA", 
+						"R-LOWA", "S-LOWA", "R-BCNT", 
+						"S-BCNT", "R-BMAX", "S-BMAX",
+					       "(state)");
+				else
+					printf("(state)\n");
+			}
 			first = 0;
 		}
 		if (Lflag && so->so_qlimit == 0)
@@ -494,10 +505,29 @@ protopr(u_long off, const char *name, int af1, int proto)
 			} /* else nothing printed now */
 #endif /* INET6 */
 		}
+		if (xflag) {
+			if (Lflag)
+				printf("%21s %6u %6u %6u %6u %6u %6u %6u %6u %6u %6u %6u %6u ",
+				       " ",
+				       so->so_rcv.sb_mcnt, so->so_snd.sb_mcnt,
+				       so->so_rcv.sb_ccnt, so->so_snd.sb_ccnt,
+				       so->so_rcv.sb_hiwat, so->so_snd.sb_hiwat,
+				       so->so_rcv.sb_lowat, so->so_snd.sb_lowat,
+				       so->so_rcv.sb_mbcnt, so->so_snd.sb_mbcnt,
+				       so->so_rcv.sb_mbmax, so->so_snd.sb_mbmax);
+			else
+				printf("%6u %6u %6u %6u %6u %6u %6u %6u %6u %6u %6u %6u ",
+				       so->so_rcv.sb_mcnt, so->so_snd.sb_mcnt,
+				       so->so_rcv.sb_ccnt, so->so_snd.sb_ccnt,
+				       so->so_rcv.sb_hiwat, so->so_snd.sb_hiwat,
+				       so->so_rcv.sb_lowat, so->so_snd.sb_lowat,
+				       so->so_rcv.sb_mbcnt, so->so_snd.sb_mbcnt,
+				       so->so_rcv.sb_mbmax, so->so_snd.sb_mbmax);
+		}
 		if (istcp && !Lflag) {
 			if (tp->t_state < 0 || tp->t_state >= TCP_NSTATES)
 				printf("%d", tp->t_state);
-                      else {
+			else {
 				printf("%s", tcpstates[tp->t_state]);
 #if defined(TF_NEEDSYN) && defined(TF_NEEDFIN)
 				/* Show T/TCP `hidden state' */
@@ -505,7 +535,7 @@ protopr(u_long off, const char *name, int af1, int proto)
 					putchar('*');
 #endif /* defined(TF_NEEDSYN) && defined(TF_NEEDFIN) */
 			}
-		}
+		} 		
 		putchar('\n');
 	}
 	if (xig != oxig && xig->xig_gen != oxig->xig_gen) {
@@ -654,6 +684,11 @@ tcp_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
 	p(tcps_sack_send_blocks, "\t%lu SACK option%s (SACK blocks) sent\n");
 	p1a(tcps_sack_sboverflow, "\t%lu SACK scoreboard overflow\n");
 
+	p(tcps_ecn_ce, "\t%lu packet%s with ECN CE bit set\n");
+	p(tcps_ecn_ect0, "\t%lu packet%s with ECN ECT(0) bit set\n");
+	p(tcps_ecn_ect1, "\t%lu packet%s with ECN ECT(1) bit set\n");
+	p(tcps_ecn_shs, "\t%lu successful ECN handshake%s\n");
+	p(tcps_ecn_rcwnd, "\t%lu time%s ECN reduced the congestion window\n");
 #undef p
 #undef p1a
 #undef p2
@@ -962,32 +997,30 @@ icmp_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
 	}
 }
 
+#ifndef BURN_BRIDGES
 /*
- * Dump IGMP statistics structure.
+ * Dump IGMP statistics structure (pre 8.x kernel).
  */
-void
-igmp_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
+static void
+igmp_stats_live_old(u_long off, const char *name)
 {
-	struct igmpstat igmpstat, zerostat;
-	size_t len = sizeof igmpstat;
+	struct oigmpstat oigmpstat, zerostat;
+	size_t len = sizeof(oigmpstat);
 
-	if (live) {
-		if (zflag)
-			memset(&zerostat, 0, len);
-		if (sysctlbyname("net.inet.igmp.stats", &igmpstat, &len,
-		    zflag ? &zerostat : NULL, zflag ? len : 0) < 0) {
-			warn("sysctl: net.inet.igmp.stats");
-			return;
-		}
-	} else
-		kread(off, &igmpstat, len);
+	if (zflag)
+		memset(&zerostat, 0, len);
+	if (sysctlbyname("net.inet.igmp.stats", &oigmpstat, &len,
+	    zflag ? &zerostat : NULL, zflag ? len : 0) < 0) {
+		warn("sysctl: net.inet.igmp.stats");
+		return;
+	}
 
 	printf("%s:\n", name);
 
-#define	p(f, m) if (igmpstat.f || sflag <= 1) \
-    printf(m, igmpstat.f, plural(igmpstat.f))
-#define	py(f, m) if (igmpstat.f || sflag <= 1) \
-    printf(m, igmpstat.f, igmpstat.f != 1 ? "ies" : "y")
+#define	p(f, m) if (oigmpstat.f || sflag <= 1) \
+    printf(m, oigmpstat.f, plural(oigmpstat.f))
+#define	py(f, m) if (oigmpstat.f || sflag <= 1) \
+    printf(m, oigmpstat.f, oigmpstat.f != 1 ? "ies" : "y")
 	p(igps_rcv_total, "\t%u message%s received\n");
 	p(igps_rcv_tooshort, "\t%u message%s received with too few bytes\n");
 	p(igps_rcv_badsum, "\t%u message%s received with bad checksum\n");
@@ -1003,6 +1036,89 @@ igmp_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
 #undef p
 #undef py
 }
+#endif /* !BURN_BRIDGES */
+
+/*
+ * Dump IGMP statistics structure.
+ */
+void
+igmp_stats(u_long off, const char *name, int af1 __unused, int proto __unused)
+{
+	struct igmpstat igmpstat, zerostat;
+	size_t len;
+
+#ifndef BURN_BRIDGES
+	if (live) {
+		/*
+		 * Detect if we are being run against a pre-IGMPv3 kernel.
+		 * We cannot do this for a core file as the legacy
+		 * struct igmpstat has no size field, nor does it
+		 * export it in any readily-available symbols.
+		 */
+		len = 0;
+		if (sysctlbyname("net.inet.igmp.stats", NULL, &len, NULL,
+		    0) < 0) {
+			warn("sysctl: net.inet.igmp.stats");
+			return;
+		}
+		if (len < sizeof(igmpstat)) {
+			igmp_stats_live_old(off, name);
+			return;
+		}
+	}
+#endif /* !BURN_BRIDGES */
+
+	len = sizeof(igmpstat);
+	if (live) {
+		if (zflag)
+			memset(&zerostat, 0, len);
+		if (sysctlbyname("net.inet.igmp.stats", &igmpstat, &len,
+		    zflag ? &zerostat : NULL, zflag ? len : 0) < 0) {
+			warn("sysctl: net.inet.igmp.stats");
+			return;
+		}
+	} else {
+		len = sizeof(igmpstat);
+		kread(off, &igmpstat, len);
+	}
+
+	if (igmpstat.igps_version != IGPS_VERSION_3) {
+		warnx("%s: version mismatch (%d != %d)", __func__,
+		    igmpstat.igps_version, IGPS_VERSION_3);
+	}
+	if (igmpstat.igps_len != IGPS_VERSION3_LEN) {
+		warnx("%s: size mismatch (%d != %d)", __func__,
+		    igmpstat.igps_len, IGPS_VERSION3_LEN);
+	}
+
+	printf("%s:\n", name);
+
+#define	p64(f, m) if (igmpstat.f || sflag <= 1) \
+    printf(m, (uintmax_t) igmpstat.f, plural(igmpstat.f))
+#define	py64(f, m) if (igmpstat.f || sflag <= 1) \
+    printf(m, (uintmax_t) igmpstat.f, pluralies(igmpstat.f))
+	p64(igps_rcv_total, "\t%ju message%s received\n");
+	p64(igps_rcv_tooshort, "\t%ju message%s received with too few bytes\n");
+	p64(igps_rcv_badttl, "\t%ju message%s received with wrong TTL\n");
+	p64(igps_rcv_badsum, "\t%ju message%s received with bad checksum\n");
+	py64(igps_rcv_v1v2_queries, "\t%ju V1/V2 membership quer%s received\n");
+	py64(igps_rcv_v3_queries, "\t%ju V3 membership quer%s received\n");
+	py64(igps_rcv_badqueries,
+	    "\t%ju membership quer%s received with invalid field(s)\n");
+	py64(igps_rcv_gen_queries, "\t%ju general quer%s received\n");
+	py64(igps_rcv_group_queries, "\t%ju group quer%s received\n");
+	py64(igps_rcv_gsr_queries, "\t%ju group-source quer%s received\n");
+	py64(igps_drop_gsr_queries, "\t%ju group-source quer%s dropped\n");
+	p64(igps_rcv_reports, "\t%ju membership report%s received\n");
+	p64(igps_rcv_badreports,
+	    "\t%ju membership report%s received with invalid field(s)\n");
+	p64(igps_rcv_ourreports,
+"\t%ju membership report%s received for groups to which we belong\n");
+        p64(igps_rcv_nora, "\t%ju V3 report%s received without Router Alert\n");
+        p64(igps_snd_reports, "\t%ju membership report%s sent\n");
+#undef p64
+#undef py64
+}
 
 /*
  * Dump PIM statistics structure.
@@ -1113,8 +1229,7 @@ inetname(struct in_addr *inp)
 	if (inp->s_addr == INADDR_ANY)
 		strcpy(line, "*");
 	else if (cp) {
-		strncpy(line, cp, sizeof(line) - 1);
-		line[sizeof(line) - 1] = '\0';
+		strlcpy(line, cp, sizeof(line));
 	} else {
 		inp->s_addr = ntohl(inp->s_addr);
 #define	C(x)	((u_int)((x) & 0xff))
