@@ -82,8 +82,8 @@ static struct nlist nl[] = {
 	{ .n_name = "_rt_tables"},
 #define	N_MRTSTAT	3
 	{ .n_name = "_mrtstat" },
-#define	N_MFCTABLE	4
-	{ .n_name = "_mfctable" },
+#define	N_MFCHASHTBL	4
+	{ .n_name = "_mfchashtbl" },
 #define	N_VIFTABLE	5
 	{ .n_name = "_viftable" },
 #define	N_IPX		6
@@ -182,6 +182,8 @@ static struct nlist nl[] = {
 	{ .n_name = "_rip6stat" },
 #define	N_SCTPSTAT	53
 	{ .n_name = "_sctpstat" },
+#define	N_MFCTABLESIZE	54
+	{ .n_name = "_mfctablesize" },
 	{ .n_name = NULL },
 };
 
@@ -333,6 +335,7 @@ int	rflag;		/* show routing tables (or routing stats) */
 int	sflag;		/* show protocol statistics */
 int	tflag;		/* show i/f watchdog timers */
 int	Wflag;		/* wide display */
+int	xflag;		/* extra information, includes all socket buffer info */
 int	zflag;		/* zero stats */
 
 int	interval;	/* repeat interval for i/f stats */
@@ -351,7 +354,7 @@ main(int argc, char *argv[])
 
 	af = AF_UNSPEC;
 
-	while ((ch = getopt(argc, argv, "AaBbdf:ghI:iLlM:mN:np:rSstuWw:z")) != -1)
+	while ((ch = getopt(argc, argv, "AaBbdf:ghI:iLlM:mN:np:rSstuWw:xz")) != -1)
 		switch(ch) {
 		case 'A':
 			Aflag = 1;
@@ -460,6 +463,9 @@ main(int argc, char *argv[])
 			interval = atoi(optarg);
 			iflag = 1;
 			break;
+		case 'x':
+			xflag = 1;
+			break;
 		case 'z':
 			zflag = 1;
 			break;
@@ -546,7 +552,8 @@ main(int argc, char *argv[])
 #endif
 		} else {
 			if (af == AF_INET || af == AF_UNSPEC)
-				mroutepr(nl[N_MFCTABLE].n_value,
+				mroutepr(nl[N_MFCHASHTBL].n_value,
+					 nl[N_MFCTABLESIZE].n_value,
 					 nl[N_VIFTABLE].n_value);
 #ifdef INET6
 			if (af == AF_INET6 || af == AF_UNSPEC)
@@ -554,7 +561,6 @@ main(int argc, char *argv[])
 					  nl[N_MIF6TABLE].n_value);
 #endif
 		}
-		ifmalist_dump();
 		exit(0);
 	}
 
@@ -766,7 +772,7 @@ static void
 usage(void)
 {
 	(void)fprintf(stderr, "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
-"usage: netstat [-AaLnSW] [-f protocol_family | -p protocol]\n"
+"usage: netstat [-AaLnSWx] [-f protocol_family | -p protocol]\n"
 "               [-M core] [-N system]",
 "       netstat -i | -I interface [-abdhntW] [-f address_family]\n"
 "               [-M core] [-N system]",
