@@ -107,6 +107,7 @@ static void TerminalStop(int);
 
 static struct bundle *SignalBundle;
 static struct prompt *SignalPrompt;
+struct libalias *la;
 
 void
 Cleanup()
@@ -327,7 +328,7 @@ main(int argc, char **argv)
   log_Open(name ? name + 1 : argv[0]);
 
 #ifndef NONAT
-  PacketAliasInit();
+  la = LibAliasInit(NULL);
 #endif
   label = ProcessArgs(argc, argv, &sw);
 
@@ -508,9 +509,11 @@ main(int argc, char **argv)
       if (!sw.fg)
         setsid();
     } else {
-      /* -direct - STDIN_FILENO gets used by physical_Open */
+      /*
+       * -direct - STDIN_FILENO gets used by physical_Open.  STDOUT_FILENO
+       * *may* get used in exec/pipe mode.
+       */
       prompt_TtyInit(NULL);
-      close(STDOUT_FILENO);
       close(STDERR_FILENO);
     }
   } else {
